.include "../inc/x16.inc"
.include "../inc/math.inc"
.include "../inc/ctl.defines.inc"
.include "../inc/print.inc"
.include "../inc/vera.inc"
.include "../inc/file.inc"

.export ctl_data
.export ctl_cur_data
.export ctl_next

.export ctl_process
.export ctl_keyboard
;.export ctl_get_key
.export ctl_create
.export ctl_init

NUM_CTLS = 20
CTL_SIZE = 16

ctl_data:	.res CTL_SIZE*NUM_CTLS
ctl_cur_data: .word 0
ctl_next:	.byte 1


ctl_normal:
	lda #ctl_state_over
	ldy #ctl_data_state
	sta (ZPW_CTL_INDEX),y

	; Exit by jumping to the control's draw function.
	; The address in ZPW_CTL_INDEX serves as the link to the data.
	ldy #ctl_data_draw_ptr
	lda (ZPW_CTL_INDEX),y
	sta $30
	iny
	lda (ZPW_CTL_INDEX),y
	sta $31
	jmp ($30)


ctl_over:
	lda ctl_cur_data+ctl_data_state

	cmp #$e0
	beq @return
	lda ZPB_MOUSE_BTNS
	cmp #1
	bne @button_right
	lda #ctl_state_pressed
	ldy #ctl_data_state
	sta (ZPW_CTL_INDEX),y

	ldy #ctl_data_mouse_left_ptr
	lda (ZPW_CTL_INDEX),y
	sta $30
	iny
	lda (ZPW_CTL_INDEX),y
	sta $31
	jmp ($30)

	@button_right:
	lda ZPB_MOUSE_BTNS
	cmp #2
	bne @return
	lda #ctl_state_pressed
	ldy #ctl_data_state
	sta (ZPW_CTL_INDEX),y

	ldy #ctl_data_mouse_right_ptr
	lda (ZPW_CTL_INDEX),y
	sta $30
	iny
	lda (ZPW_CTL_INDEX),y
	sta $31
	jmp ($30)

	@return:
	rts


ctl_key_pressed:
	ldy #ctl_data_draw_ptr
	lda (ZPW_CTL_INDEX),y
	sta $30
	iny
	lda (ZPW_CTL_INDEX),y
	sta $31
	jmp ($30)
	rts


ctl_pressed:
	lda ZPB_MOUSE_BTNS
	bne @return
	lda #ctl_state_over
	ldy #ctl_data_state
	sta (ZPW_CTL_INDEX),y

	ldy #ctl_data_draw_ptr
	lda (ZPW_CTL_INDEX),y
	sta $30
	iny
	lda (ZPW_CTL_INDEX),y
	sta $31
	jmp ($30)

	@return:
	rts


ctl_select:
	ldy #(ctl_data_state+1)
	lda (ZPW_CTL_INDEX),y
	cmp ZPB_CTL_FOCUS
	bne @no_key_pressed
	lda ZPB_CTL_KEY
	beq @no_key_pressed
	jsr ctl_key_pressed
	rts

	@no_key_pressed:
	math_MOUSE_IN_BOX_ADDR
	beq @continue

	ldy #ctl_data_state
	lda (ZPW_CTL_INDEX),y

	cmp #$c0
	beq @state_normal
	cmp #$d0
	beq @state_over
	cmp #$e0
	beq @state_pressed
	rts

	@continue:	; Mouse is off control, Reset to normal state.
	; Unless it's already normal... then return.
	ldy #ctl_data_state
	lda (ZPW_CTL_INDEX),y
	cmp #$c0
	bne @set_normal
	rts

	@set_normal:
	lda #$c0
	ldy #ctl_data_state
	sta (ZPW_CTL_INDEX),y
	ldy #ctl_data_draw_ptr
	lda (ZPW_CTL_INDEX),y
	sta $30
	iny
	lda (ZPW_CTL_INDEX),y
	sta $31
	jmp ($30)
	rts

	@state_normal:
	jsr ctl_normal
	rts

	@state_over:
	jsr ctl_over
	rts

	@state_pressed:
	jsr ctl_pressed
	rts


; TODO move these ZP addresses out of general use and block them with
; the rest of the CTL registers.
ctl_process:
	lda #0
	sta $54

	@loop:
	lda $54
	sta ZPW_CTL_INDEX
	stz ZPW_CTL_INDEX+1
	math_BIN_MULTIPLY_16 ZPW_CTL_INDEX, #4
	mem_LOAD_16 ctl_data, $34 
	math_ADD_16 $34, ZPW_CTL_INDEX	; ZPW_CTL_INDEX now holds the base address for the cur data

	ldy #ctl_data_state
	lda (ZPW_CTL_INDEX),y
	beq @continue

	jsr ctl_select	

	@continue:
	inc $54
	lda $54
	cmp #19
	bne @loop
	rts


ctl_keyboard:
	lda ZPB_CTL_FOCUS
	cmp #0
	beq @continue
	jsr GETIN
	cmp #0
	beq @return
;	sec
;	sbc #$40
	sta ZPB_CTL_KEY

;	jsr GETIN
;	cmp #0
;	beq @return
;	sec
;	sbc #$40
;	sta (ZPW_EDIT_FOCUS)
;	math_INC_16 ZPW_EDIT_FOCUS
	
;	vera_SET_TEXT_ADDR_XY #2, #2
;	print_STRING file_name, #$c7

	bra @return

	@continue:
	jsr GETIN
	cmp #0
	beq @return

	@return:
	rts


;ctl_get_key:
;	; A holds index of calling control.
;	cmp ZPB_CTL_FOCUS
;	bne @return
;	lda ZPB_CTL_KEY
;	@return:
;	rts


ctl_create:	
	; Compute offset from ctl_data and place in ctl_cur_data 

	lda $40		; Index
	sta ZPW_CTL_INDEX
	stz ZPW_CTL_INDEX+1
	math_BIN_MULTIPLY_16 ZPW_CTL_INDEX, #4
	mem_LOAD_16 ctl_data, $34 
	math_ADD_16 $34, ZPW_CTL_INDEX	; ZPW_CTL_INDEX now holds the base address for the cur data

	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $42, #ctl_data_loc_x
	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $44, #ctl_data_loc_y
	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $46, #ctl_data_width
	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $48, #ctl_data_height
	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $4a, #ctl_data_draw_ptr
	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $4c, #ctl_data_mouse_left_ptr
	mem_XFER_16_ZPINDY ZPW_CTL_INDEX, $4e, #ctl_data_mouse_right_ptr

	; Set index and initial state.
	lda #ctl_state_over
	ldy #ctl_data_state
	sta (ZPW_CTL_INDEX),y
	iny
	lda $40
	sta (ZPW_CTL_INDEX),y
	; Display initial state.
	rts


ctl_init:
	rts
