.include "../inc/x16.inc"
.include "../inc/math.inc"
.include "../inc/vera.inc"
.include "../inc/ctl.inc"
.include "../inc/ctl.defines.inc"
.include "../inc/print.inc"
.include "../inc/palette.inc"
.include "../inc/sprite.inc"
.include "../inc/strip.inc"

file_bios_name:			.byte "test.spr       "
file_name_maxlen:		.byte 15
file_name_len:			.byte 8
file_name_cursor:		.byte 0
file_name_cursor_char:	.byte 0
file_name:		 		.byte $14,$05,$13,$14,$2e,$13,$10,$12,$20,$20,$20,$20,$20,$20,$20,0

file_str_load: .byte $0c,$0f,$01,$04,0
file_str_save: .byte $13,$01,$16,$05,0

.export file_bios_name
.export file_insert_char

.export file_set_cursor_on
.export file_set_cursor_off
.export file_set_index_to_frame
.export file_clear_cur_frame
.export file_update_cur_frame
.export file_init


file_load_sprite:
	; Load the sprite data file into VRAM_sprite_data.
	lda #1
	ldx #8
	ldy #0
	jsr SETLFS

	lda file_name_len
	ldx #<file_bios_name
	ldy #>file_bios_name
	jsr SETNAM

	lda #3	;0 = CPU RAM; 2 = VERA bank 0; 3 = VERA bank 1 
	ldx #<VRAM_sprite_data
	ldy #>VRAM_sprite_data
	jsr LOAD
	rts


file_load_sprite_info:
	; Load the sprite's data structure to ram.

	mem_SET_24_INDIRECT VRAM_sprite_data, ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	lda VERA_data0
	sta sprite_bpp

	lda VERA_data0
	sta sprite_frame_w

	lda VERA_data0
	sta sprite_frame_h

	lda VERA_data0
	sta sprite_num_frames
	rts


file_load_sprite_palette:
	; Load the 16 colors in the sprite to the VRAM_palette.
	; palette entries $0f to $ff (2 bytes per, so offset $1e0.)

	mem_SET_24_INDIRECT (VRAM_sprite_data+4), ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	; Loop counter, f0 to ff. On rollover, loop is done.
	; Also used to indicate the current index to edit.
	lda #$f0
	sta ZPB_R0	

	mem_SET_24_INDIRECT (VRAM_palette+$1e0), ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 1, $10	;Addr1, stride 1
				
	@loop:
	lda VERA_data0	;First byte of palette data.
	sta VERA_data1
	lda VERA_data0	;Second byte of palette data.
	sta VERA_data1
	inc ZPB_R0
	bne @loop
	rts


file_save_sprite_info:
	; Save the sprite's data structure to ram.

	mem_SET_24_INDIRECT VRAM_sprite_data, ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	lda sprite_bpp
	sta VERA_data0

	lda sprite_frame_w
	sta VERA_data0

	lda sprite_frame_h
	sta VERA_data0

	lda sprite_num_frames
	sta VERA_data0
	rts


file_save_sprite_palette:
	; Save the 16 colors in the sprite from the VRAM_palette.
	; palette entries $0f to $ff (2 bytes per, so offset $1e0.)

	mem_SET_24_INDIRECT (VRAM_palette+$1e0), ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	; Loop counter, f0 to ff. On rollover, loop is done.
	lda #$f0
	sta ZPB_R0	

	mem_SET_24_INDIRECT (VRAM_sprite_data+4), ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 1, $10	;Addr1, stride 1
				
	@loop:
	lda VERA_data0	;First byte of palette data.
	sta VERA_data1
	lda VERA_data0	;Second byte of palette data.
	sta VERA_data1
	inc ZPB_R0
	bne @loop
	rts


file_save_sprite:
	lda #1
	ldx #8
	ldy #1
	jsr SETLFS

	lda file_name_len
	ldx #<file_bios_name
	ldy #>file_bios_name
	jsr SETNAM

	jsr OPEN
	ldx #1
	jsr CHKOUT

	mem_SET_24_INDIRECT VRAM_sprite_data, ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	lda #0		; Storage address, ignored by VRAM load.
	jsr CHROUT
	jsr CHROUT

	; Figure out the exact file size.
	mem_LOAD_16 36, ZPW_R0
	lda sprite_num_frames
	sta ZPB_R0
	@frame_count:
	math_ADD_16_INDIRECT 512, ZPW_R0
	dec ZPB_R0
	beq @loop	
	bra @frame_count

	@loop:
	lda VERA_data0
	jsr CHROUT
	math_DEC_16 ZPW_R0
	lda ZPW_R0
	bne @loop
	lda ZPW_R0+1
	bne @loop

	lda #1
	jsr CLOSE
	rts


file_name_draw_over:
	vera_SET_TEXT_ADDR_XY #2, #2
	print_STRING file_name, #$c7
	rts


file_name_draw_normal:
	; Can't happen for an edit control while there is a focus.
	; TODO must check against THIS control.
	lda ZPB_CTL_FOCUS
	bne @return

	vera_SET_TEXT_ADDR_XY #2, #2
	print_STRING file_name, #$c1
	
	@return:
	rts


file_set_cursor_on:
	lda ZPB_CTL_FOCUS
	beq @return	
	lda file_name_cursor_char
	cmp #0
	bne @return
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_cursor, ZPW_R0
	lda (ZPW_R0)
	sta file_name_cursor_char
	clc
	adc #$80
	sta (ZPW_R0)
	vera_SET_TEXT_ADDR_XY #2, #2
	print_STRING file_name, #$c7
	@return:
	rts


file_set_cursor_off:
	lda ZPB_CTL_FOCUS
	beq @return	
	lda file_name_cursor_char
	cmp #0
	beq @return
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_cursor, ZPW_R0
	lda file_name_cursor_char
	sta (ZPW_R0)
	stz file_name_cursor_char
	vera_SET_TEXT_ADDR_XY #2, #2
	print_STRING file_name, #$c7
	@return:
	rts


file_set_bios_name:
	mem_LOAD_16 file_name, ZPW_R0
	mem_LOAD_16 file_bios_name, ZPW_R1
	lda file_name_maxlen
	sta ZPB_R0

	@loop:
	lda (ZPW_R0)
	cmp #$01 ; char 'A'
	bmi @no_add
	cmp #$1a ; char 'Z'
	bpl @no_add
	clc
	adc #$40
	@no_add:
	sta(ZPW_R1)
	math_INC_16 ZPW_R0
	math_INC_16 ZPW_R1
	dec ZPB_R0
	bne @loop
	rts


file_set_name_length:
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_maxlen, ZPW_R0
	lda file_name_maxlen
	sta ZPB_R0

	@loop:
	dec ZPB_R0
	math_DEC_16 ZPW_R0
	lda (ZPW_R0)
	cmp #$20
	beq @set_length
	lda ZPB_R0
	beq @return
	bne @loop

	@set_length:
	lda ZPB_R0
	sta file_name_len
	bra @loop

	@return:
	rts


file_inc_char:
	lda file_name_maxlen
	dec
	cmp file_name_cursor
	beq @return

	jsr file_set_cursor_off
	stz ZPB_CTL_KEY
	inc file_name_cursor

	@return:
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_cursor, ZPW_R0
	jsr file_set_cursor_on
	rts


file_dec_char:
	lda file_name_cursor
	beq @return

	jsr file_set_cursor_off
	stz ZPB_CTL_KEY
	dec file_name_cursor

	@return:
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_cursor, ZPW_R0
	jsr file_set_cursor_on
	rts


file_insert_char:
	lda file_name_cursor
	inc
	inc
	cmp file_name_maxlen
	bmi @continue
	jsr file_add_char	; Use add char for the last 2 chars.
	rts

	@continue:
	; Incoming character is in ZPB_R1
	lda ZPB_R1
	cmp #$41 ; char 'A'
	bmi @no_sub
	cmp #$7a ; char 'Z'
	bpl @no_sub
	sec
	sbc #$40
	@no_sub:
	sta ZPB_R1

	jsr file_set_cursor_off
	lda file_name_maxlen
	dec
	dec
	sta ZPB_R2
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 ZPB_R2, ZPW_R0
	mem_XFER_16 ZPW_R0, ZPW_R1
	math_INC_16 ZPW_R1

	@loop:
	lda (ZPW_R0)
	sta (ZPW_R1)
	math_DEC_16 ZPW_R0
	math_DEC_16 ZPW_R1
	lda ZPB_R2
	dec 
	sta ZPB_R2
	cmp file_name_cursor
	bne @loop

	lda (ZPW_R0)
	sta (ZPW_R1)
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_cursor, ZPW_R0
	lda ZPB_R1
	sta (ZPW_R0)
	inc file_name_cursor

	stz ZPB_CTL_KEY
	jsr file_name_draw_over
	jsr file_set_cursor_on
	rts


file_add_char:
	jsr file_set_cursor_off
	lda ZPB_R1
	cmp #$41 ; char 'A'
	bmi @no_sub
	cmp #$7a ; char 'Z'
	bpl @no_sub
	sec
	sbc #$40
	@no_sub:
	sta (ZPW_R0)
	stz ZPB_CTL_KEY
	jsr file_name_draw_over
	jsr file_inc_char
	rts


file_add_delete:
	lda file_name_maxlen
	dec
	cmp file_name_cursor
	beq @return
	
	jsr file_set_cursor_off
	lda file_name_cursor
	inc
	sta ZPB_R2

	@loop:
	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 ZPB_R2, ZPW_R0
	lda (ZPW_R0)
	sta ZPB_R3
	lda #$20
	sta (ZPW_R0)
	math_DEC_16 ZPW_R0
	lda ZPB_R3
	sta (ZPW_R0)
	lda file_name_maxlen
	dec
	cmp ZPB_R2
	beq @return

	inc ZPB_R2
	bra @loop

	@return:	
	stz ZPB_CTL_KEY
	jsr file_name_draw_over
	rts


file_delete_last_char:
	jsr file_set_cursor_off

	; If not on the last character, just return.
	lda file_name_maxlen
	dec
	cmp file_name_cursor
	bne @return

	; If the last char is already a space, return.
	lda (ZPW_R0)
	cmp #$20
	beq @return

	; Delete the last character. (Set it to space.)
	lda #$20
	sta (ZPW_R0)
	stz ZPB_CTL_KEY
	lda #0 ; Tell calling function you deleted the char.
	@return:
	rts


file_add_backspace:
	jsr file_delete_last_char
	cmp #0
	beq @return

	jsr file_dec_char
	jsr file_set_cursor_off
	lda #$20
	sta (ZPW_R0)
	stz ZPB_CTL_KEY
	jsr file_name_draw_over
	
	@return:
	jsr file_set_cursor_on
	rts


file_add_return:
	jsr file_set_cursor_off
	stz ZPB_CTL_FOCUS
	stz ZPB_CTL_KEY
	jsr file_name_draw_normal
	stz file_name_cursor
	jsr file_set_bios_name
	jsr file_set_name_length
	rts


file_name_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $45	; The control's index.
	cmp ZPB_CTL_FOCUS
	bne @continue
	lda ZPB_CTL_KEY ; The next key (If any.)
	cmp #0
	beq @continue

	mem_LOAD_16 file_name, ZPW_R0
	math_ADD_16_BY_8 file_name_cursor, ZPW_R0
	lda ZPB_CTL_KEY
	sta ZPB_R1
	; Specific chars first.
	cmp #$19
	beq @delete
	cmp #$14
	beq @backspace
	cmp #$0d
	beq @return
	cmp #$9d
	beq @left_arrow
	cmp #$1d
	beq @right_arrow
	; By default, all chars as if they're printable.
	; Choose between insert and overwrite mode here.
	jsr file_insert_char	; insert mode
;	jsr file_add_char		; overwrite mode
	rts

	@delete:
	jsr file_add_delete
	rts

	@backspace:
	jsr file_add_backspace
	rts

	@return:
	jsr file_add_return
	rts

	@left_arrow:
	jsr file_dec_char
	rts

	@right_arrow:
	jsr file_inc_char
	rts

	@continue:
	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr file_name_draw_over
	rts

	@normal:
	jsr file_name_draw_normal
	rts


file_name_mouse_left:
	mem_CTL_XFER_16 #ctl_data_state, $44
	lda $45
	sta ZPB_CTL_FOCUS
	jsr file_set_cursor_on
	rts


file_name_mouse_right:
	rts


file_load_draw_over:
	vera_SET_TEXT_ADDR_XY #4, #4
	print_STRING file_str_load, #$c7
	rts


file_load_draw_normal:
	vera_SET_TEXT_ADDR_XY #4, #4
	print_STRING file_str_load, #$c1
	rts


file_load_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
;	mem_CTL_XFER_16 #ctl_data_loc_x, $40
;	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr file_load_draw_over
	rts
	@normal:
	jsr file_load_draw_normal
	rts


file_load_mouse_left:
	jsr file_load_sprite
	jsr file_load_sprite_info
	jsr file_load_sprite_palette

	jsr palette_get_index
	jsr sprite_load_editor
	jsr sprite_draw_editor
	jsr strip_load_all_frames

	jsr strip_bpp_draw_normal
	jsr strip_width_draw_normal
	jsr strip_height_draw_normal
	jsr strip_frames_draw_normal

	vera_SET_TEXT_ADDR_XY #24, #21
	print_HEX_CHAR palette_r, #$b1
	vera_SET_TEXT_ADDR_XY #25, #21
	print_HEX_CHAR palette_g, #$b1
	vera_SET_TEXT_ADDR_XY #26, #21
	print_HEX_CHAR palette_b, #$b1
	rts


file_load_mouse_right:
	rts


file_save_draw_over:
	vera_SET_TEXT_ADDR_XY #4, #6
	print_STRING file_str_save, #$c7
	rts


file_save_draw_normal:
	vera_SET_TEXT_ADDR_XY #4, #6
	print_STRING file_str_save, #$c1
	rts


file_save_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_loc_x, $40
	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr file_save_draw_over
	rts
	@normal:
	jsr file_save_draw_normal
	rts


file_save_mouse_left:
	jsr file_save_sprite_info
	jsr file_save_sprite_palette
	jsr file_save_sprite
	rts


file_save_mouse_right:
	rts


file_set_index_to_frame:
	sta ZPB_R0
	mem_SET_24_INDIRECT (VRAM_sprite_data+36), ZP24_R0
	stz ZPB_R1
	@loop:
	lda ZPB_R0
	cmp ZPB_R1
	beq @skip_add_frame
	math_ADD_24_INDIRECT 512, ZP24_R0
	inc ZPB_R1
	bra @loop
	@skip_add_frame:
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	@continue:
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1
	rts


file_clear_cur_frame:
	jsr file_set_index_to_frame

	mem_LOAD_16 512, ZPW_R0
	@loop:
	lda #0
	sta VERA_data0
	math_DEC_16 ZPW_R0
	lda ZPW_R0
	bne @loop
	lda ZPW_R0+1
	bne @loop
	rts


file_update_cur_frame:
	jsr file_set_index_to_frame

	mem_LOAD_16 sprite_editor, ZPW_R1
	mem_LOAD_16 512, ZPW_R2

	@loop_pixels:
	lda (ZPW_R1)
	and #$0f ; Strip off palette page.
	clc
	asl
	asl
	asl
	asl
	sta ZPB_R0
	math_INC_16 ZPW_R1
	lda (ZPW_R1)
	and #$0f ; Strip off palette page.
	adc ZPB_R0
	sta ZPB_R0
	math_INC_16 ZPW_R1

	lda ZPB_R0
	sta VERA_data0 	
	math_DEC_16 ZPW_R2
	lda ZPW_R2
	bne @loop_pixels
	lda ZPW_R2+1
	bne @loop_pixels
	rts


file_init:
	ctl_CREATE 16, 16, 96, 8, file_name_draw, file_name_mouse_left, file_name_mouse_right

	ctl_CREATE 32, 32, 32, 8, file_load_draw, file_load_mouse_left, file_load_mouse_right

	ctl_CREATE 32, 48, 32, 8, file_save_draw, file_save_mouse_left, file_save_mouse_right

	lda #0
	jsr file_clear_cur_frame
	rts
