.include "../inc/x16.inc"

.export BIN
.export BCD
.export HTD_IN
.export HTD_OUT

.export math_mult_32
.export math_add_24
.export math_convert_to_bcd
.export math_bcd_16
.export math_mouse_in_box

math_mult_32:
; $30,$31,$32,$33	result
; $34,$35,$36,$37	offset
; $38,$39		operand 1
; $3a,$3b		operand 2
; $3c,$3d		temp_word
; $3e			temp_byte
	lda $3a		; <operand 2
	sta $30		; <result
	lda $3b		; >operand 2
	sta $31		; >result
	stz $32		; ^result
	stz $33		; -result
	stz $34		; <offset
	stz $35		; >offset
	stz $36		; ^offset
	stz $37		; -offset
	lda $38		; <operand 1
	sta $3c		; <temp_word
	lda $39		; >operand 1
	sta $3d		; >temp_word
	beq @op1_byte
	ldy #16

	@op1_word_exp_loop:
	dey
	asl $3d		; >temp_word
	bcc @op1_word_exp_loop
	sty $3e		; temp_byte
	lda #16
	sec
	sbc $3e		; temp_byte
	tax

	@op1_word_remainder_loop:	
	lsr $3d		; >temp_word
	dex
	bne @op1_word_remainder_loop
	bra @shift

	@op1_byte:
	lda $3c		; <temp_word
	bne @set_op1_byte_exp
	stz $30		; <result
	stz	$31		; >result
	jmp @return

	@set_op1_byte_exp:
	ldy	#8

	@op1_byte_exp_loop:
	dey
	asl $3c		; <temp_word
	bcc @op1_byte_exp_loop
	sty $3e		; temp_byte
	lda #8
	sec
	sbc $3e		; temp_byte
	tax

	@op1_byte_remainder_loop:
	lsr $3c		; <temp_word
	dex
	bne @op1_byte_remainder_loop

; $30,$31,$32,$33	result
; $34,$35,$36,$37	offset
; $38,$39		operand 1
; $3a,$3b		operand 2
; $3c,$3d		temp_word
; $3e			temp_byte

	@shift:
	cpy #0
	beq @remainder
	dey
	lsr $3d		; >temp_word
	ror	$3c		; <temp_word
	bcc @shift_result
	ldx #0
	clc
	php			; push status to stack

	@add_offset:
	plp			; pull status from stack
	lda $34,x	; <offset	 
	adc $30,x	; <result
	sta	$34,x	; <offset
	php
	inx
	cpx #4
	bne @add_offset
	plp

	@shift_result:
	asl $30		; <result
	rol $31		; >result
	rol	$32		; ^result
	rol	$33		; -result
	bra @shift

	@remainder:
	ldx #0
	clc
	php

	@add_result:
	plp
	lda $30,x		; <result
	adc $34,x		; <offset
	sta $30,x		; <result
	php
	inx
	cpx #4
	bne @add_result
	plp

	@return:
	rts


math_add_24:
; $30,$31,$32,$33	result
; $34,$35,$36		operand 1
; $37,$38,$39		operand 2
	clc
	lda $34
	adc $37
	sta $30
	lda $35
	adc $38
	sta $31
	lda $36
	adc $39
	sta $32
	stz $33
	rts


HTD_IN:     .res 2
HTD_OUT:    .res 4       ; low byte first

TABLE:      .word    1, 2, 4, 8, 16H, 32H, 64H, 128H
                ; (Word directive puts low byte first.)

math_convert_to_bcd:
HTD:    SED             ; Output gets added up in decimal.
        STZ HTD_OUT     ; Inititalize output word as 0.
        STZ HTD_OUT+1   ; (NMOS 6502 will need LDA#0, STA ...)

        LDX #0EH        ; $E is 14 for 2x7 bits.  (0-7 is 8 positions.)
 loop:  ASL HTD_IN      ; Look at next high bit.  If it's 0,
        BCC @htd1       ; don't add anything to the output for this bit.
        LDA HTD_OUT     ; Otherwise get the running output sum
        CLC
        ADC TABLE,X     ; and add the appropriate value for this bit
        STA HTD_OUT     ; from the table, and store the new sum.
        LDA HTD_OUT+1   ; After low byte, do high byte.
        ADC TABLE+1,X
        STA HTD_OUT+1

 @htd1: DEX             ; Go down to next bit value to loop again.
        DEX
        BPL loop        ; If still not done, go back for another loop.

        CLD
        RTS


; Convert an 16 bit binary value to BCD
;
; This function converts a 16 bit binary value into a 24 bit BCD. It
; works by transferring one bit a time from the source and adding it
; into a BCD value that is being doubled on each iteration. As all the
; arithmetic is being done in BCD the result is a binary to decimal
; conversion. All conversions take 915 clock cycles.
;
; See BINBCD8 for more details of its operation.
;
; Andrew Jacobs, 28-Feb-2004

; The data to convert / output
BIN:		.word 0
BCD:		.res 3

math_bcd_16:
		SED		; Switch to decimal mode
		LDA #0		; Ensure the result is clear
		STA BCD+0
		STA BCD+1
		STA BCD+2
		LDX #16	; The number of source bits

CNVBIT:	ASL BIN+0	; Shift out one bit
		ROL BIN+1
		LDA BCD+0	; And add into result
		ADC BCD+0
		STA BCD+0
		LDA BCD+1	; propagating any carry
		ADC BCD+1
		STA BCD+1
		LDA BCD+2	; ... thru whole result
		ADC BCD+2
		STA BCD+2
		DEX		; And repeat for next bit
		BNE CNVBIT
		CLD		; Back to binary

		rts		; All Done.


; $40,$41		screen_x
; $42,$43		screen_y
; $44,$45		width
; $46,$47		height
; $48,$49		math result
; A is the return. '0' is "Nope!" Non-zero is "Mouse is in the box!"
math_mouse_in_box:
	; Subtract mouse_x from screen_x... if plus, can't be in box!
	sec
	lda ZPW_MOUSE_X
	sbc $40
	sta $48
	lda ZPW_MOUSE_X+1
	sbc $41
	sta $49
	bmi @return_nope

	; Subtract mouse_y from screen_y... if plus, can't be in box!
	sec
	lda ZPW_MOUSE_Y
	sbc $42
	sta $48
	lda ZPW_MOUSE_Y+1
	sbc $43
	sta $49
	bmi @return_nope

	clc
	lda $40
	adc $44
	sta $40
	lda $41
	adc $45
	sta $41
	sec
	lda ZPW_MOUSE_X
	sbc $40
	sta $48
	lda ZPW_MOUSE_X+1
	sbc $41
	sta $49
	bpl @return_nope

	clc
	lda $42
	adc $46
	sta $42
	lda $43
	adc $47
	sta $43
	sec
	lda ZPW_MOUSE_Y
	sbc $42
	sta $48
	lda ZPW_MOUSE_Y+1
	sbc $43
	sta $49
	bpl @return_nope


	lda #01
	rts
	@return_nope:
	lda #0
	rts