.include "../inc/x16.inc"
.include "../inc/math.inc"
.include "../inc/vera.inc"
.include "../inc/draw.inc"
.include "../inc/print.inc"
.include "../inc/palette.inc"
.include "../inc/strip.inc"
.include "../inc/file.inc"

sprite_str_Frame: .byte $06,$12,$01,$0d,$05,$20,$30,$31,0

; VERA address offsets.
sprite_anim_box_offset = 31+(196*320)
sprite_anim_block_offset = 32+(96*320)
sprite_editor_box_offset = 167+(31*320)
sprite_editor_offset = 168+(32*320)

.export sprite_editor
;.export line_start
.export sprite_bpp
.export sprite_frame_w
.export sprite_frame_h
.export sprite_num_frames
.export sprite_start_frame
.export sprite_end_frame
.export sprite_cur_frame

.export sprite_load_editor
.export sprite_remove_pips
.export sprite_add_pip_text
.export sprite_add_pips
.export sprite_draw_editor
.export sprite_init
.export sprite_show_debug

; Sprite info
sprite_bpp:			.byte 4
sprite_frame_w:		.byte 32
sprite_frame_h:		.byte 32
sprite_num_frames:	.byte 1
sprite_start_frame:	.byte 0
sprite_end_frame:	.byte 5
sprite_cur_frame:	.byte 0

sprite_editor: .res 1024

;line_start: .word 0
;line_data: .byte 0

pips_active: .byte 0
pips_x: .byte 0,0
pips_y: .byte 0,0

pips_text_x = 30
pips_text_y = 2


sprite_load_editor:
	; Loop: iterate through all bytes in 4 bpp data in VRAM_sprite_data,
	; Expand into 8 bpp and store in sprite editor.
	; TODO delete file_buffer and index into the current frame in
	; VRAM_sprite_data.
	mem_LOAD_16 0, ZPW_R0	;Pixel count.
	mem_LOAD_16 sprite_editor, ZPW_R1

	lda sprite_cur_frame
	jsr file_set_index_to_frame

;	mem_SET_24_INDIRECT (VRAM_sprite_data+36), ZP24_R0
;	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	@loop:
	lda VERA_data0
	sta ZPB_R0
	lsr
	lsr
	lsr
	lsr
	ora #$f0	; Add in the default palette page.
	sta (ZPW_R1)
	math_INC_16 ZPW_R1
	lda ZPB_R0		; Do the second pixel.
	ora #$f0
	sta (ZPW_R1)
	math_INC_16 ZPW_R1

	; Increment zp 16 bit counter and loop.
	math_INC_16 ZPW_R0
	cmp #$02	; msb
	bne @loop
	lda ZPW_R0
	cmp #$00	; lsb 
	bne @loop
	rts


sprite_remove_pips:
	lda #pips_text_x
	sta $34
	lda #pips_text_y
	sta $35
	stz $3b
	jsr print_char
	stz $3b
	jsr print_next_char
	stz $3b
	jsr print_next_char
	stz $3b
	jsr print_next_char
	stz $3b
	jsr print_next_char
	rts


sprite_add_pip_text:
	mem_XFER_16 ZPW_MOUSE_X, $48
	math_BIN_DIVIDE_16 $48, 2
	sec
	lda $48
	sbc #$2a
	sta pips_x

	mem_XFER_16 ZPW_MOUSE_Y, $48
	math_BIN_DIVIDE_16 $48, 2
	sec
	lda $48
	sbc #$08
	sta pips_y
	bra @continue;

	@return:
	rts

	@continue:
	vera_SET_TEXT_ADDR_XY #pips_text_x, #pips_text_y
	print_BCD_BYTE pips_x, #$b1
	print_CHAR #$2c, #$b1
	print_BCD_BYTE pips_y, #$b1
	rts


sprite_add_pips:
	math_MOUSE_IN_BOX_IMM 168, 32, 128, 128
	bne @continue
	jsr sprite_remove_pips
	rts

	@continue:
	jsr sprite_add_pip_text

	lda ZPB_MOUSE_BTNS
	cmp #00		; No buttons clicked.
	beq @return
	lda #<sprite_editor
	sta $48
	lda #>sprite_editor
	sta $49

	lda #32
	sta $38
	stz $39
	lda pips_y
	sta $3a
	stz $3b
	jsr math_mult_32
	math_ADD_16 $30, $48
	math_ADD_16 pips_x, $48

	lda ZPB_MOUSE_BTNS
	cmp #01		; Left button clicked.
	bne @right_button
	lda palette_cur_color
	sta ($48)
	jsr sprite_draw_editor
	lda sprite_cur_frame
	jsr file_update_cur_frame
	jsr strip_load_all_frames
	rts

	@right_button:
	lda ZPB_MOUSE_BTNS
	cmp #02		; Middle button clicked.
	bne @return;
	lda palette_back_color
	sta ($48)
	jsr sprite_draw_editor
	lda sprite_cur_frame
	jsr file_update_cur_frame
	jsr strip_load_all_frames
	rts

	@return:
	rts


;sprite_draw_editor:
;	vera_SET_ADDR VRAM_bitmap, 0
;	mem_LOAD_24 sprite_editor_offset, ZP24_VERA_ADDR1 
;	jsr


sprite_draw_editor:
	mem_LOAD_24 sprite_editor_offset, $30 
	
	lda #<sprite_editor
	sta $48
	lda #>sprite_editor
	sta $49

	ldy #32
	ldx #32
	@loop:
	vera_SET_OFFSET_24 $30
	lda ($48)
	sta $37
	draw_BLOCK 4, 4
	math_INC_16 $48
	math_ADD_OFFSET 4
	dex
	bne @loop
	math_ADD_OFFSET 1152
	ldx #32
	dey
	bne @loop
	rts


sprite_init:
	vera_SET_OFFSET sprite_anim_box_offset
	draw_BOX 34, 34, $09

	vera_SET_OFFSET sprite_editor_box_offset
	draw_BOX 130, 130, $09

	vera_SET_TEXT_ADDR_XY #21, #2
	print_STRING sprite_str_Frame, #$b1

	jsr sprite_draw_editor
	rts


sprite_show_debug:
	lda ZPB_CTL_KEY
	sta BIN
	stz BIN+1
	jsr math_bcd_16

;	print_CHAR #$2c, #$b8

	lda BCD
	and #$0f
	ora #$30
	sta ZPB_R0
	print_CHAR ZPB_R0, #$b5
rts

	lda sprite_start_frame
	sta BIN
	stz BIN+1
	jsr math_bcd_16

	print_CHAR #$2c, #$b8

	lda BCD
	and #$0f
	ora #$30
	sta ZPB_R0
	print_CHAR ZPB_R0, #$b5

	lda sprite_end_frame
	sta BIN
	stz BIN+1
	jsr math_bcd_16

	print_CHAR #$2c, #$b8

	lda BCD
	and #$0f
	ora #$30
	sta ZPB_R0
	print_CHAR ZPB_R0, #$b5
	rts	
