.include "../inc/x16.inc"
.include "../inc/math.inc"
.include "../inc/vera.inc"
.include "../inc/ctl.inc"
.include "../inc/ctl.defines.inc"
.include "../inc/draw.inc"
.include "../inc/print.inc"
.include "../inc/sprite.inc"
.include "../inc/file.inc"


.export strip_box_draw_normal
.export strip_load_all_frames
.export strip_copy_cur_frame
.export strip_bpp_draw_normal
.export strip_width_draw_normal
.export strip_height_draw_normal
.export strip_frames_draw_normal
.export strip_init

; VERA address offsets.
sprite_strip_outline_offset = 95+(31*320)
sprite_strip_outline1_offset = 95+(64*320)
sprite_strip_outline2_offset = 95+(97*320)
sprite_strip_outline3_offset = 95+(130*320)
sprite_strip_outline4_offset = 95+(163*320)
sprite_strip_outline5_offset = 95+(196*320)

sprite_strip_box_offset: .dword 0
sprite_strip_box0_offset: .dword 96+(32*320)
sprite_strip_box1_offset: .dword 96+(65*320)
sprite_strip_box2_offset: .dword 96+(98*320)
sprite_strip_box3_offset: .dword 96+(131*320)
sprite_strip_box4_offset: .dword 96+(164*320)
sprite_strip_box5_offset: .dword 96+(197*320)

sprite_slide_box_offset = 132+(42*320)

strip_str_bpp:		.byte $20,$02,$10,$10,$20,$20,$0
strip_str_width:	.byte $17,$09,$04,$14,$08,$20,$0
strip_str_height:	.byte $08,$05,$09,$07,$08,$14,$0
strip_str_frames:	.byte $06,$12,$01,$0d,$05,$13,$0
strip_str_reset:	.byte $12,$05,$13,$05,$14,$0


strip_load_all_frames:
	lda sprite_start_frame
	sta ZPB_R0
	mem_LOAD_16 sprite_strip_box_offset, ZPW_R0
	mem_LOAD_16 4, ZPW_R1

	@loop:
	; Use zero page address indirection to get the effective
	; address from the table of sprite strip box offsets.
	math_ADD_16 ZPW_R1, ZPW_R0
	ldy #0
	lda (ZPW_R0),y
	sta ZPW_R2
	iny
	lda (ZPW_R0),y
	sta ZPW_R2+1
	stz ZPW_R3

	mem_SET_24_INDIRECT VRAM_bitmap, ZP24_R0
	math_ADD_24 ZPW_R2, ZP24_R0  

	sec
	lda ZPB_R0
	sbc sprite_num_frames
	bpl @clear

	jsr strip_copy_cur_frame
	bra @continue

	@clear:
	jsr strip_clear_cur_frame

	@continue:
	lda sprite_end_frame
	cmp ZPB_R0
	beq @return
	inc ZPB_R0
	bra @loop

	@return:
	rts


strip_copy_cur_frame:
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	; Index the strip frame to match the frame in ZPB_R0.
	mem_SET_24_INDIRECT (VRAM_sprite_data+36), ZP24_R1
	stz ZPB_R1
	@loop:
	lda ZPB_R0
	cmp ZPB_R1
	beq @skip_add_frame
	math_ADD_24_INDIRECT 512, ZP24_R1
	inc ZPB_R1
	bra @loop
	@skip_add_frame:
	vera_SET_VRAM_ADDR ZP24_R1, 1, $10	;Addr1, stride 1

	lda #32
	sta ZPB_R1
	lda #16
	sta ZPB_R2
	@loop_y:
	lda VERA_data1
	sta ZPB_R3
	lsr
	lsr
	lsr
	lsr
	ora #$f0
	sta VERA_data0
	lda ZPB_R3
	ora #$f0
	sta VERA_data0
	dec ZPB_R2
	bne @loop_y 

	math_ADD_24_INDIRECT 320, ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1
	lda #16
	sta ZPB_R2
	dec ZPB_R1
	bne @loop_y
	rts


strip_clear_cur_frame:
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1

	; Reset the frame to the background color.
	lda #32
	sta ZPB_R1
	lda #16
	sta ZPB_R2
	@loop_y:
	lda #$0b
	sta VERA_data0
	sta VERA_data0
	dec ZPB_R2
	bne @loop_y 

	math_ADD_24_INDIRECT 320, ZP24_R0
	vera_SET_VRAM_ADDR ZP24_R0, 0, $10	;Addr0, stride 1
	lda #16
	sta ZPB_R2
	dec ZPB_R1
	bne @loop_y
	rts


strip_box_set_cur_frame:
	sec
	lda ZPW_MOUSE_Y
	sbc #31
	sta ZPW_R6
	lda ZPW_MOUSE_Y+1
	sbc #0
	sta ZPW_R6+1

	; Frame is ZPW_R6/33 (0 to 5)
	; Hmm. Strip anything above 32... remainder is Frame
	lda ZPW_R6
	lsr
	lsr
	lsr
	lsr
	lsr
	sta sprite_cur_frame
	rts


strip_box_draw_over:
	vera_SET_OFFSET sprite_strip_outline_offset
	draw_BOX 34, 199, $07
	rts


strip_box_draw_normal:
	vera_SET_OFFSET sprite_strip_outline_offset
	draw_BOX 34, 199, $09

	; TODO fix the line, box, and block macros to use current ZP
	; Must still use the old $30 based system until then.
	vera_SET_OFFSET sprite_strip_outline_offset

	mem_LOAD_16 (33*320), ZPW_R6
	stz ZPB_R7
	@loop:
	lda sprite_cur_frame
	cmp ZPB_R7
	beq @done
	vera_ADD_OFFSET_16 ZPW_R6
	inc ZPB_R7
	bra @loop

	@done:
;	draw_BOX 34, 34, $08
	rts


strip_box_draw:
	; ctl_JSR_ON_STATE ctl_state_over, strip_box_draw_over
	; ctl_JSR_ON_STATE ctl_state_normal, strip_box_draw_normal

	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_state, ZPW_R7

	lda ZPW_R7
	cmp #ctl_state_normal
	beq @normal
	jsr strip_box_draw_over
	rts
	@normal:
	jsr strip_box_draw_normal
	rts


strip_box_mouse_left:
	; Figure out which frame was clicked 
	jsr strip_box_set_cur_frame

	jsr sprite_load_editor
	jsr sprite_draw_editor
	vera_SET_TEXT_ADDR_XY #27, #2

	lda sprite_cur_frame
	inc
	sta BIN
	stz BIN+1
	jsr math_bcd_16
	lda BCD
	lsr
	lsr
	lsr
	lsr
	ora #$30
	sta $40
	print_CHAR $40, #$b1
	lda BCD
	and #$0f
	ora #$30
	sta ZPB_R0
	print_CHAR ZPB_R0, #$b1

	rts


strip_box_mouse_right:
	rts


strip_bpp_draw_over:
	vera_SET_TEXT_ADDR_XY #2, #9
	print_STRING strip_str_bpp, #$c7
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_bpp, #$b1
	rts


strip_bpp_draw_normal:
	vera_SET_TEXT_ADDR_XY #2, #9
	print_STRING strip_str_bpp, #$c1
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_bpp, #$b1
	rts


strip_bpp_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_loc_x, $40
	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr strip_bpp_draw_over
	rts
	@normal:
	jsr strip_bpp_draw_normal
	rts


strip_bpp_mouse_left:
	lda sprite_bpp
	cmp #08
	beq @return
	lda #08
	sta sprite_bpp
	@return:
	rts


strip_bpp_mouse_right:
	lda sprite_bpp
	cmp #04
	beq @return
	lda #04
	sta sprite_bpp
	@return:
	rts


strip_width_draw_over:
	vera_SET_TEXT_ADDR_XY #2, #11
	print_STRING strip_str_width, #$c7
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_frame_w, #$b1
	rts


strip_width_draw_normal:
	vera_SET_TEXT_ADDR_XY #2, #11
	print_STRING strip_str_width, #$c1
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_frame_w, #$b1
	rts


strip_width_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_loc_x, $40
	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr strip_width_draw_over
	rts
	@normal:
	jsr strip_width_draw_normal
	rts


strip_width_mouse_left:
	lda sprite_frame_w
	cmp #64
	beq @return
	clc
	asl
	sta sprite_frame_w
	@return:
	rts


strip_width_mouse_right:
	lda sprite_frame_w
	cmp #8
	beq @return
	lsr
	sta sprite_frame_w
	@return:
	rts


strip_height_draw_over:
	vera_SET_TEXT_ADDR_XY #2, #13
	print_STRING strip_str_height, #$c7
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_frame_h, #$b1
	rts


strip_height_draw_normal:
	vera_SET_TEXT_ADDR_XY #2, #13
	print_STRING strip_str_height, #$c1
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_frame_h, #$b1
	rts


strip_height_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_loc_x, $40
	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr strip_height_draw_over
	rts
	@normal:
	jsr strip_height_draw_normal
	rts


strip_height_mouse_left:
	lda sprite_frame_h
	cmp #64
	beq @return
	clc
	asl
	sta sprite_frame_h
	@return:
	rts


strip_height_mouse_right:
	lda sprite_frame_h
	cmp #8
	beq @return
	lsr
	sta sprite_frame_h
	@return:
	rts


strip_frames_draw_over:
	vera_SET_TEXT_ADDR_XY #2, #15
	print_STRING strip_str_frames, #$c7
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_num_frames, #$b1
	rts


strip_frames_draw_normal:
	vera_SET_TEXT_ADDR_XY #2, #15
	print_STRING strip_str_frames, #$c1
	print_CHAR #$20, #$b1
	print_BCD_BYTE sprite_num_frames, #$b1
	rts


strip_frames_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_loc_x, $40
	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr strip_frames_draw_over
	rts
	@normal:
	jsr strip_frames_draw_normal
	rts


strip_frames_mouse_left:
	lda sprite_num_frames
	cmp #6
	beq @return
	inc
	sta sprite_num_frames
	dec
	jsr file_clear_cur_frame
	jsr strip_load_all_frames
	@return:
	rts


strip_frames_mouse_right:
	lda sprite_num_frames
	cmp #1
	beq @return
	dec
	sta sprite_num_frames
	jsr strip_load_all_frames
	@return:
	rts


strip_reset_draw_over:
	vera_SET_TEXT_ADDR_XY #3, #18
	print_STRING strip_str_reset, #$c7
	rts


strip_reset_draw_normal:
	vera_SET_TEXT_ADDR_XY #3, #18
	print_STRING strip_str_reset, #$c1
	rts


strip_reset_draw:
	; ZPW_CTL_INDEX contains the base address to the control's data.
	mem_CTL_XFER_16 #ctl_data_loc_x, $40
	mem_CTL_XFER_16 #ctl_data_loc_y, $42
	mem_CTL_XFER_16 #ctl_data_state, $44

	lda $44
	cmp #ctl_state_normal
	beq @normal
	jsr strip_reset_draw_over
	rts
	@normal:
	jsr strip_reset_draw_normal
	rts


strip_reset_mouse_left:
	rts


strip_reset_mouse_right:
	rts


strip_init:
	ctl_CREATE 95, 31, 34, 199, strip_box_draw, strip_box_mouse_left, strip_box_mouse_right

	ctl_CREATE 16, 72, 40, 8, strip_bpp_draw, strip_bpp_mouse_left, strip_bpp_mouse_right

	ctl_CREATE 16, 88, 40, 8, strip_width_draw, strip_width_mouse_left, strip_width_mouse_right

	ctl_CREATE 16, 104, 40, 8, strip_height_draw, strip_height_mouse_left, strip_height_mouse_right

	ctl_CREATE 16, 120, 40, 8, strip_frames_draw, strip_frames_mouse_left, strip_frames_mouse_right

	ctl_CREATE 24, 144, 40, 8, strip_reset_draw, strip_reset_mouse_left, strip_reset_mouse_right

	vera_SET_OFFSET sprite_strip_outline_offset
	draw_BOX 34, 199, $09

	vera_SET_OFFSET sprite_strip_outline1_offset
	draw_LINE $10, 33, $09

	vera_SET_OFFSET sprite_strip_outline2_offset
	draw_LINE $10, 33, $09

	vera_SET_OFFSET sprite_strip_outline3_offset
	draw_LINE $10, 33, $09

	vera_SET_OFFSET sprite_strip_outline4_offset
	draw_LINE $10, 33, $09

	vera_SET_OFFSET sprite_strip_outline5_offset
	draw_LINE $10, 33, $09

	; Initial frame
;	vera_SET_OFFSET sprite_strip_outline_offset
;	draw_BOX 34, 34, $08

;	vera_SET_OFFSET sprite_slide_box_offset
;	draw_BOX 10, 172, $09

	jsr strip_load_all_frames
	rts
