
.import BIN
.import BCD
.import HTD_IN
.import HTD_OUT

.import math_mult_32
.import math_add_24
.import math_convert_to_bcd
.import math_bcd_16
.import math_mouse_in_box


.macro mem_LOAD_16 num, loc
	lda #<num
	sta loc
	lda #>num
	sta loc+1
	.endmacro


.macro mem_LOAD_24 num, loc
	lda #<num
	sta loc
	lda #>num
	sta loc+1
	lda #^num
	sta loc+2
	.endmacro


.macro mem_XFER_16 loc1, loc2
	lda loc1
	sta loc2
	lda loc1+1
	sta loc2+1
	.endmacro


.macro mem_CTL_XFER_16 index, dest
	ldy index
	lda (ZPW_CTL_INDEX),y
	sta dest
	iny
	lda (ZPW_CTL_INDEX),y
	sta dest+1
	.endmacro


.macro math_ADD_16 zpw_operand, zpw_result
	clc
	lda zpw_result
	adc zpw_operand
	sta zpw_result
	lda zpw_result+1
	adc zpw_operand+1
	sta zpw_result+1
	.endmacro


.macro math_ADD_16_BY_8 byte_operand, word_result
	clc
	lda word_result
	adc byte_operand
	sta word_result
	lda word_result+1
	adc #0
	sta word_result+1
	.endmacro


.macro math_ADD_16_INDIRECT operand, zpw_result
	clc
	lda zpw_result
	adc #<operand
	sta zpw_result
	lda zpw_result+1
	adc #>operand
	sta zpw_result+1
	.endmacro


.macro math_ADD_24 zp24_operand, zp24_result
	clc
	lda zp24_result
	adc zp24_operand
	sta zp24_result

	lda zp24_result+1
	adc zp24_operand+1
	sta zp24_result+1

	lda zp24_result+2
	adc zp24_operand+2
	sta zp24_result+2
	.endmacro


.macro math_ADD_24_INDIRECT operand, zp24_result
	clc
	lda zp24_result
	adc #<operand
	sta zp24_result

	lda zp24_result+1
	adc #>operand
	sta	zp24_result+1

	lda zp24_result+2
	adc #^operand
	sta	zp24_result+2
	.endmacro


.macro mem_XFER_16_ZPINDY dest, source, index
	ldy index
	lda source
	sta (dest),y
	iny
	lda source+1
	sta (dest),y
	.endmacro

;.macro mem_IMM_TO_ADDR_16 source, dest
;.macro mem_ADDR_TO_ADDR_16 source, dest
;.macro mem_ADDR_TO_ZPINDY_16 source, dest, index

.macro mem_ZPINDY_TO_ADDR_16 source, index, dest
	ldy index
	lda (source),y
	sta dest
	iny
	lda (source),y
	sta dest+1
	.endmacro


.macro mem_SET_24_INDIRECT value, zp24
	lda #<value
	sta zp24
	lda #>value
	sta zp24+1
	lda #^value
	sta zp24+2
	.endmacro


.macro math_BIN_MULTIPLY_24 num_bits, zp24
	phx
	ldx num_bits
	:
	clc
	lda zp24
	asl
	sta zp24
	lda zp24+1
	rol
	sta zp24+1
	lda zp24+2
	rol
	sta zp24+2
	dex
	bne :-
	plx
	.endmacro


.macro mem_XFER_24 loc1, loc2
	lda loc1
	sta loc2
	lda loc1+1
	sta loc2+1
	lda loc1+2
	sta loc2+2
	.endmacro


.macro math_multiply op1, op2
	lda #<op1
	sta ZP_PTR+8
	lda #>op1
	sta ZP_PTR+9
	lda #<op2
	sta ZP_PTR+10
	lda #>op2
	sta ZP_PTR+11
	jsr math_mult_32
	.endmacro


.macro math_add op1, op2
	lda #<op1
	sta ZP_PTR+4
	lda #>op1
	sta ZP_PTR+5
	lda #^op1
	sta ZP_PTR+6
	lda #<op2
	sta ZP_PTR+7
	lda #>op2
	sta ZP_PTR+8
	lda #^op2
	sta ZP_PTR+9
	jsr math_add_24
	.endmacro


.macro math_ADD_OFFSET_by_8 op1
	clc
	lda $30
	adc #op1
	sta $30
	lda $31
	adc #0
	sta	$31
	lda $32
	adc #0
	sta	$32
	.endmacro


.macro math_ADD_OFFSET op1
	clc
	lda $30
	adc #<op1
	sta $30
	lda $31
	adc #>op1
	sta	$31
	lda $32
	adc #^op1
	sta	$32
	.endmacro


.macro math_BIN_DIVIDE_16 addr16, div
	ldx #div
	:
	clc
	lda addr16+1
	ror
	sta addr16+1
	lda addr16
	ror
	sta addr16
	dex
	bne :-
	.endmacro


.macro math_BIN_MULTIPLY_16 addr16, oper
	phx
	ldx oper
	:
	clc
	lda addr16
	asl
	sta addr16
	lda addr16+1
	rol
	sta addr16+1
	dex
	bne :-
	plx
	.endmacro


.macro math_DEC_16 loc
	sec
	lda loc
	sbc #$01
	sta loc
	lda loc+1
	sbc #$00
	sta loc+1
;	bne @dec_16_continue
;	dec loc+1
;	@dec_16_continue:
	.endmacro


.macro math_INC_16 loc
	clc
	lda loc
	adc #1
	sta loc
	lda loc+1
	adc #0
	sta loc+1
	.endmacro


.macro math_chain_add op2
	lda $30
	sta $30+4	
	lda $31
	sta $30+5	
	lda $32
	sta $30+6	
	lda #<op2
	sta $30+7
	lda #>op2
	sta $30+8
	lda #^op2
	sta $30+9
	jsr math_add_24
	.endmacro


.macro math_bmp_offset loc_x, loc_y, width
	mem_LOAD_16 loc_x, $38
	mem_LOAD_16 loc_y, $3a
	mem_LOAD_16 width, $44
	mem_XFER_16 $38, $40
	mem_XFER_16 $3a, $42
	jsr math_mult_32
	mem_LOAD_24 VERA_bitmap, $34
	.endmacro


; $40,$41		screen_x
; $42,$43		screen_y
; $44,$45		width
; $46,$47		height
; A is the return. '0' is "Nope!"" Non-zero is "Mouse is in the box!"
.macro math_MOUSE_IN_BOX_IMM loc_x, loc_y, width, height
	lda #<loc_x
	sta $40
	lda #>loc_x
	sta $41
	lda #<loc_y
	sta $42
	lda #>loc_y
	sta $43
	lda #<width
	sta $44
	lda #>width
	sta $45
	lda #<height
	sta $46
	lda #>height
	sta $47

	jsr math_mouse_in_box
	.endmacro


.macro math_MOUSE_IN_BOX_ADDR

;ctl_data_loc_x = 0
;ctl_data_loc_y = 2
;ctl_data_width = 4
;ctl_data_height = 6
;ctl_data_state = 8
;ctl_data_draw_ptr = 10
;ctl_data_mouse_left_ptr = 12
;ctl_data_mouse_right_ptr = 14

	mem_ZPINDY_TO_ADDR_16 ZPW_CTL_INDEX, #ctl_data_loc_x, $40
	mem_ZPINDY_TO_ADDR_16 ZPW_CTL_INDEX, #ctl_data_loc_y, $42
	mem_ZPINDY_TO_ADDR_16 ZPW_CTL_INDEX, #ctl_data_width, $44
	mem_ZPINDY_TO_ADDR_16 ZPW_CTL_INDEX, #ctl_data_height, $46

;	mem_XFER_16 loc_x, $40
;	mem_XFER_16 loc_y, $42
;	mem_XFER_16 width, $44
;	mem_XFER_16 height, $46

	jsr math_mouse_in_box
	.endmacro
